package com.example.lab.DataBase.Models;

import jakarta.persistence.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Entity
public class Product {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;
    private String name;
    private float price;

    @OneToMany
    private List<CountProduct> countProducts;

    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "productCategory_fk")
    private ProductCategory productCategory;

    public Product(){}

    public Product(String name, float price, ProductCategory productCategory){
        this.name = name;
        this.price = price;
        this.productCategory = productCategory;
        this.countProducts = new ArrayList<>();
    }

    public ProductCategory getProductCategory() {
        return productCategory;
    }

    public void setProductCategory(ProductCategory productCategory) {
        this.productCategory = productCategory;
        if(productCategory == null){
            return;
        }
        if (!productCategory.getProducts().contains(this)){
            productCategory.AddProduct(this);
        }
    }

    public Long getId() {
        return id;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setPrice(float price) {
        this.price = price;
    }

    public float getPrice() {
        return price;
    }

    @PreRemove
    public void deleteThis(){
        //deleteAllCarts();
        this.productCategory.getProducts().remove(this);
    }


    public List<CountProduct> getCountProducts(){
        return countProducts;
    }

    public void addCountProduct(CountProduct countProduct){
        countProducts.add(countProduct);
        if (countProduct.getProduct() != this) {
            countProduct.setProduct(this);
        }
    }

    public void deleteCountProduct(CountProduct countProduct){
        countProducts.remove(countProduct);
        if (countProduct.getProduct() == this) {
            countProduct.setProduct(null);
        }
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) return true;
        if (obj == null || getClass() != obj.getClass()) return false;
        Product product = (Product) obj;
        return Objects.equals(id, product.id);
    }

    @Override
    public int hashCode(){
        return Objects.hashCode(id);
    }

}
