package com.example.lab.DataBase.Services;

import com.example.lab.DataBase.Models.Customer;
import com.example.lab.DataBase.Models.Product;
import jakarta.persistence.*;
import jakarta.transaction.Transactional;
import org.springframework.stereotype.Service;
import com.example.lab.DataBase.Models.Cart;
import java.util.List;

@Service
public class CartService {
    @PersistenceContext
    private EntityManager em;

    private final ProductService productService;
    private final CountProductService countProductService;

    public CartService(ProductService productService, CountProductService countProductService){
        this.productService = productService;
        this.countProductService = countProductService;
    }

    @Transactional
    public Cart addCart(Customer customer){
        Cart cart = new Cart(customer);
        cart.getCustomer().setCart(cart);
        em.persist(cart);
        return cart;
    }

    @Transactional()
    public Cart getCart(Long id){
        Cart cart = em.find(Cart.class, id);
        if (cart == null){
            throw new EntityNotFoundException(String.format("Error: Cart id %s not found", id));
        }
        return cart;
    }

    @Transactional
    public List<Cart> getAllCarts(){
        return em.createQuery("from Cart", Cart.class).getResultList();
    }

    @Transactional
    public Cart deleteCart(Long id){
        final Cart cart = getCart(id);
        em.remove(cart);
        return cart;
    }
    @Transactional
    public void deleteAllCarts(){
        for (var cart:
             getAllCarts()) {
            cart.deleteThis();
            em.remove(cart);
        }
    }

    @Transactional
    public Product addProduct(Long cartId, Long productId){
        Product product = productService.getProduct(productId);
        Cart cart = getCart(cartId);
        var countProduct = countProductService.getCountProduct(productId, cartId);
        if(countProduct == null) countProduct = countProductService.addCountProduct(product, cart);
        countProduct.incrementAmount();
        em.persist(cart);
        return product;
    }

    @Transactional
    public Product deleteProduct(Long cartId, Long productId){
        Product product = productService.getProduct(productId);
        Cart cart = getCart(cartId);
        var countProduct = countProductService.getCountProduct(productId, cartId);
        if(countProduct == null) return null;
        countProductService.deleteCountProduct(productId,cartId);
        em.persist(cart);
        return product;
    }

    @Transactional
    public Product reduceProduct(Long cartId, Long productId){
        Product product = productService.getProduct(productId);
        Cart cart = getCart(cartId);
        var countProduct = countProductService.getCountProduct(productId, cartId);
        if(countProduct == null) return null;
        countProduct.decrementAmount();
        em.persist(countProduct);
        return product;
    }
}
