package com.example.lab.DataBase.Services;

import com.example.lab.DataBase.Models.*;
import jakarta.persistence.*;
import jakarta.transaction.Transactional;
import org.springframework.stereotype.Service;
import com.example.lab.DataBase.Models.Product;

import java.util.List;

@Service
public class CountProductService {
    @PersistenceContext
    private EntityManager em;

    @Transactional
    public CountProduct addCountProduct(Product product,
                                         Cart cart){
        CountProduct countProduct = new CountProduct(cart, product);
        em.persist(countProduct);
        return countProduct;
    }

    @Transactional
    public CountProduct getCountProduct(long productId, long cartId){
        var count = getAllCountProducts();
        var countProduct = count.stream().filter(x -> x.getProduct().getId() == productId
                && x.getCart().getId() == cartId ).findFirst();
        if(countProduct.isEmpty()) return null;
        else return countProduct.get();
    }

    @Transactional
    public CountProduct incrementProduct (long productId, long cartId){
        CountProduct countProduct = getCountProduct(productId, cartId);
        countProduct.incrementAmount();
        return em.merge(countProduct);
    }

    @Transactional
    public CountProduct decrementProduct (long productId, long cartId){
        CountProduct countProduct = getCountProduct(productId, cartId);
        countProduct.decrementAmount();
        return em.merge(countProduct);
    }

    @Transactional
    public CountProduct deleteCountProduct(long productId, long cartId){
        final CountProduct countProduct = getCountProduct(productId, cartId);
        em.remove(countProduct);
        return countProduct;
    }

    @Transactional
    public void deleteAll(){
        var list = getAllCountProducts();
        for (var cp:
             list) {
            cp.deleteThis();
            em.remove(cp);
        }
    }

    @Transactional
    public List<CountProduct> getAllCountProducts(){
        return em.createQuery("from CountProduct", CountProduct.class).getResultList();
    }
}