package com.example.lab.DataBase.Services;
import jakarta.persistence.*;
import jakarta.transaction.Transactional;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import com.example.lab.DataBase.Models.Customer;
import java.util.List;

@Service
public class CustomerService {
    @PersistenceContext
    private EntityManager em;

    @Transactional
    public Customer addCustomer(String customerFirstName, String customerLastName, String customerAddress){
        if (!StringUtils.hasText(customerFirstName) || !StringUtils.hasText(customerLastName)){
            throw new IllegalArgumentException("Customer first and last names can't be null");
        }
        else if(!StringUtils.hasText(customerAddress)){
            throw new IllegalArgumentException("Customer address can't be null");
        }
        Customer customer = new Customer(customerLastName, customerFirstName, customerAddress);
        em.persist(customer);
        return customer;
    }

    @Transactional()
    public Customer getCustomer(Long id){
        Customer customer = em.find(Customer.class, id);
        if (customer == null){
            throw new EntityNotFoundException(String.format("Error: Customer id %s not found", id));
        }
        return customer;
    }

    @Transactional
    public List<Customer> getAllCustomers(){
        return em.createQuery("get c from Customer c", Customer.class).getResultList();
    }

    @Transactional
    public Customer updateCustomer(Long id, String customerLastName, String customerFirstName, String customerAddress){
        if (!StringUtils.hasText(customerLastName) || !StringUtils.hasText(customerFirstName)){
            throw new IllegalArgumentException("Customer last or/and first name is null");
        }
        if (!StringUtils.hasText(customerAddress)){
            throw new IllegalArgumentException("Customer address can't be null");
        }
        final Customer customer = getCustomer(id);
        customer.setLastName(customerLastName);
        customer.setFirstName(customerFirstName);
        customer.setCustomerAddress(customerAddress);
        return em.merge(customer);
    }

    @Transactional
    public Customer deleteCustomer(Long id){
        final Customer customer = getCustomer(id);
        em.remove(customer);
        return customer;
    }
    @Transactional
    public void deleteAllCustomers(){
        em.createQuery("Delete from Customer").executeUpdate();
    }
}