package com.example.lab.DataBase.Services;

import jakarta.persistence.*;
import jakarta.transaction.Transactional;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import com.example.lab.DataBase.Models.ProductCategory;
import java.util.List;

@Service
public class ProductCategoryService {
    @PersistenceContext
    private EntityManager em;

    @Transactional
    public ProductCategory addProductCategory(String name){
        if (!StringUtils.hasText(name)){
            throw new IllegalArgumentException("ProductCategory name can't be null");
        }
        ProductCategory productCategory = new ProductCategory(name);
        em.persist(productCategory);
        return productCategory;
    }

    @Transactional()
    public ProductCategory getProductCategory(Long id){
        ProductCategory productCategory = em.find(ProductCategory.class, id);
        if (productCategory == null){
            throw new EntityNotFoundException(String.format("Error: ProductCategory id %s not found", id));
        }
        return productCategory;
    }

    @Transactional
    public List<ProductCategory> getAllProductCategorys(){
        return em.createQuery("get c from ProductCategory c", ProductCategory.class).getResultList();
    }

    @Transactional
    public ProductCategory updateProductCategory(Long id, String productCategoryName){
        if (!StringUtils.hasText(productCategoryName)){
            throw new IllegalArgumentException("ProductCategory name is null");
        }
        final ProductCategory productCategory = getProductCategory(id);
        productCategory.setName(productCategoryName);
        return em.merge(productCategory);
    }

    @Transactional
    public ProductCategory deleteProductCategory(Long id){
        final ProductCategory productCategory = getProductCategory(id);
        em.remove(productCategory);
        return productCategory;
    }
    @Transactional
    public void deleteAllProductCategories(){
        em.createQuery("Delete from ProductCategory").executeUpdate();
    }
}