package com.example.lab.DataBase.Services;

import com.example.lab.DataBase.Models.ProductCategory;
import jakarta.persistence.*;
import jakarta.transaction.Transactional;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import com.example.lab.DataBase.Models.Product;
import java.util.List;

@Service
public class ProductService {
    @PersistenceContext
    private EntityManager em;

    @Transactional
    public Product addProduct(String name, float price, ProductCategory productCategory){
        if (!StringUtils.hasText(name)){
            throw new IllegalArgumentException("Product first and last names can't be null");
        }
        else if(price <= 0){
            throw new IllegalArgumentException("Product price cant be under 0");
        }
        Product product = new Product(name, price, productCategory);
        product.getProductCategory().AddProduct(product);
        em.persist(product);
        return product;
    }

    @Transactional()
    public Product getProduct(Long id){
        Product product = em.find(Product.class, id);
        if (product == null){
            throw new EntityNotFoundException(String.format("Error: Product id %s not found", id));
        }
        return product;
    }

    @Transactional
    public List<Product> getAllProducts(){
        return em.createQuery("from Product", Product.class).getResultList();
    }

    @Transactional
    public Product updateProduct(Long id, String name, float price){
        if (!StringUtils.hasText(name)){
            throw new IllegalArgumentException("Product name cant be null");
        }
        else if(price <= 0){
            throw new IllegalArgumentException("Product price cant be under 0");
        }
        final Product product = getProduct(id);
        product.setName(name);
        product.setPrice(price);
        return em.merge(product);
    }

    @Transactional
    public Product deleteProduct(Long id){
        final Product product = getProduct(id);
        em.remove(product);
        return product;
    }
    @Transactional
    public void deleteAllProducts(){
        for (var product:
             getAllProducts()) {
            product.deleteThis();
            em.remove(product);
        }
    }
}