package com.example.lab;

import com.example.lab.DataBase.Models.*;
import com.example.lab.DataBase.Services.CartService;
import com.example.lab.DataBase.Services.CustomerService;
import com.example.lab.DataBase.Services.ProductCategoryService;
import com.example.lab.DataBase.Services.ProductService;
import com.example.lab.DataBase.Services.CountProductService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

@SpringBootTest
public class DBTests {
    @Autowired
    CartService cartService;
    @Autowired
    CustomerService customerService;
    @Autowired
    ProductCategoryService productCategoryService;
    @Autowired
    ProductService productService;
    @Autowired
    CountProductService countProductService;


    @Test
    void testProductCategory(){
        cleanAll();

        ProductCategory productCategory = productCategoryService.addProductCategory("exampleCategory");
        Assertions.assertEquals("exampleCategory", productCategory.getName());
        productCategoryService.updateProductCategory(productCategory.getId(), "updExample");
        Assertions.assertEquals("updExample", productCategoryService
                .getProductCategory(productCategory.getId()).getName());

        Assertions.assertEquals("updExample", productCategoryService
                .deleteProductCategory(productCategory.getId()).getName());

        cleanAll();
    }

    @Test
    void testProduct() {
        cleanAll();

        ProductCategory productCategory = productCategoryService.addProductCategory("exampleCategory");
        Product product = productService.addProduct("example", (float) 50, productCategory);

        Assertions.assertEquals("example", product.getName());
        Assertions.assertEquals("exampleCategory", product.getProductCategory().getName());
        Assertions.assertEquals("example", productService.getProduct(product.getId()).getName());
        Assertions.assertEquals((float) 50, productService.getProduct(product.getId()).getPrice());
        productService.updateProduct(product.getId(), "newExample", (float) 60);
        Assertions.assertEquals("newExample", productService.getProduct(product.getId()).getName());
        Assertions.assertEquals((float) 60, productService.getProduct(product.getId()).getPrice());

        Assertions.assertEquals("newExample", productService.deleteProduct(product.getId()).getName());

        cleanAll();
    }

    @Test
    void testCustomer(){
        cleanAll();
        Customer customer = customerService.addCustomer("Ivan",
                "Ivanov", "cityExample");
                Assertions.assertEquals("Ivan", customerService
                .getCustomer(customer.getId()).getFirstName());
        Assertions.assertEquals("Ivanov", customerService
                .getCustomer(customer.getId()).getLastName());
        Assertions.assertEquals("cityExample", customerService
                .getCustomer(customer.getId()).getCustomerAddress());
        customerService.updateCustomer(customer.getId(),"Vaskin",
                "Vasilii", "newAddress");
        Assertions.assertEquals("Vasilii", customerService
                .getCustomer(customer.getId()).getFirstName());
        Assertions.assertEquals("Vaskin", customerService
                .getCustomer(customer.getId()).getLastName());
        Assertions.assertEquals("newAddress", customerService
                .getCustomer(customer.getId()).getCustomerAddress());

        Assertions.assertEquals("Vasilii", customerService
                .deleteCustomer(customer.getId()).getFirstName());

        cleanAll();
    }

    @Test
    void testCart(){
        cleanAll();

        ProductCategory productCategory = productCategoryService
                .addProductCategory("exampleCategory");
        Product product = productService
                .addProduct("exampleProduct", (float)50, productCategory );
        Customer customer = customerService.addCustomer("Ivan",
                "Ivanov", "cityExample");
        Cart cart = cartService.addCart(customer);

        Assertions.assertEquals(product,cartService.addProduct(cart.getId(), product.getId()));
        Assertions.assertEquals(cart,cartService.getCart(cart.getId()));
        Assertions.assertEquals(product, cartService.reduceProduct(cart.getId(),product.getId()));
        Assertions.assertEquals(product, cartService.deleteProduct(cart.getId(),product.getId()));
        Assertions.assertEquals(cart,cartService.deleteCart(cart.getId()));
        cleanAll();
    }

    public void cleanAll(){
        countProductService.deleteAll();
        productService.deleteAllProducts();
        cartService.deleteAllCarts();
        customerService.deleteAllCustomers();
        productCategoryService.deleteAllProductCategories();
    }
}
